@@ -1,7 +1,7 @@
 use strict;
 package Parse::CSV;
 {
-  $Parse::CSV::VERSION = '2.03';
+  $Parse::CSV::VERSION = '2.04';
 }
 
 =pod
@@ -12,7 +12,7 @@ Parse::CSV - Highly flexible CSV parser for large files
 
 =head1 VERSION
 
-version 2.03
+version 2.04
 
 =head1 SYNOPSIS
 
@@ -77,16 +77,15 @@ or drop records you don't need.
 =head2 Writing Filters
 
 A L<Parse::CSV> filter is a subroutine reference that is passed the raw
-record as C<$_>, and should C<return> the alternative or modified record
-to return to the user.
+record as C<$_>, and should C<return> the alternative or modified record.
 
 The null filter (does not modify or drop any records) looks like the
 following.
 
   sub { $_ };
 
-A filter which reversed the order of the columns (assuming they are
-passed as an array) might look like the following.
+A filter which reversed the order of the columns (assuming the parser
+is in array mode) might look like the following.
 
   sub { return [ reverse @$_ ] };
 
@@ -94,7 +93,7 @@ To drop the record, you should return C<undef> from the filter. The
 parser will then keep pulling and parsing new records until one
 passes the filter.
 
-  # Only keep records where foo is true
+  # Only keep records where the 'foo' field is true
   sub { $_->{foo} ? $_ : undef }
 
 To signal an error, throw an exception
@@ -122,18 +121,24 @@ use Params::Util 1.00 ();
 
 =head2 new
 
-The C<new> constructor creates and initialise a new CSV parser.
+The C<new> constructor creates and initialises a new CSV parser.  It
+returns a new L<Parse::CSV> object, or throws an exception (dies) on
+error.  It accepts a number of params:
 
-It takes a number of params.
+=over 4
 
-To specify the CSV data source, you should provide either the C<file>
+=item C<file>
+
+=item C<handle>
+
+To specify the CSV data source, provide either the C<file>
 param, which should be the name of the file to read, or the C<handle>
 param, which should be a file handle to read instead.
 
-The actual parsing is done using L<Text::CSV_XS>. Any of its
-constructor/parsing params can also be provided to this C<new> method,
-and they will be passed on.
+=item C<csv_attr>
 
+Any parameter for L<Text::CSV_XS>'s constructor can also be provided
+to this C<new> method, and they will be passed on to it.
 Alternatively, they can be passed as a single C<HASH> reference as the
 C<csv_attr> param. For example:
 
@@ -145,24 +150,32 @@ C<csv_attr> param. For example:
       },
   );
 
-An optional C<names> param can be provided, which should be an array
-reference containing the names of the columns in the CSV file.
+=item C<names>
+
+An optional C<names> param can be provided, which should either be an
+array reference containing the names of the columns:
 
   $parser = Parse::CSV->new(
       file  => 'file.csv',
       names => [ 'col1', 'col2', 'col3' ],
   );
 
-If the C<names> param is provided, the parser will map the columns to a
+or a true value that's not a reference, indicating that the column
+names will be read from the first line of the input:
+
+  $parser = Parse::CSV->new(
+      file  => 'file.csv',
+      names => 1,
+  );
+
+If the C<names> param is provided, the parser will map each line to a
 hash where the keys are the field names provided, and the values are the
 values found in the CSV file.
 
 If the C<names> param is B<not> provided, the parser will return simple
 array references of the columns.
 
-If the C<names> param is true and not a reference, the names will be
-automatically determined by reading the first line of the CSV file and
-using those values as the field names.
+=item C<filter>
 
 The optional C<filter> param will be used to filter the records if
 provided. It should be a C<CODE> reference or any otherwise callable
@@ -170,7 +183,7 @@ scalar, and each value parsed (either array reference or hash reference)
 will be passed to the filter to be changed or converted into an object,
 or whatever you wish.
 
-Returns a new L<Parse::CSV> object, or throws an exception (dies) on error.
+=back
 
 =cut
 
@@ -282,6 +295,16 @@ following.
       # Handle errors...
   }
 
+NOTE: currently the L<fields> and L<string> methods can be used to
+access the most recently-read row (as an array ref or a formatted
+string) after using C<fetch>.  However, this contradicts the
+documentation for L<Text::CSV_XS>, which says those methods should be
+"meaningless" after calling C<getline> (which C<fetch()> internally
+uses to read the input).  Keeping the current behavior also incurs a
+speed & memory penalty.  Therefore, relying on L<fields> and L<string>
+to return the current data after C<fetch()> is deprecated and will
+(probably) be removed in a future release.
+
 =cut
 
 sub fetch {
@@ -291,24 +314,19 @@ sub fetch {
 	# iterate till we get something.
 	while ( my $row = $self->_getline ) {
 		# Turn the array ref into a hash if needed
-		my $rv    = undef;
-		my $names = $self->{names};
-		my @cols  = @$row;
-		if ( $names ) {
+		my $rv;
+		if ( $self->{names} ) {
 			$rv = {};
-			foreach ( 0 .. $#$names ) {
-				$rv->{ $names->[$_] } = $cols[$_];
-			}
+			@{$rv}{@{$self->{names}}} = @$row;
 		} else {
-			$rv = \@cols;
+			$rv = $row;
 		}
 
 		# Just return for simple uses
 		return $rv unless $self->{filter};
 
 		# Filter if needed
-		local $_ = $rv;
-		$rv = eval { $self->{filter}->() };
+		$rv = eval { local $_ = $rv; $self->{filter}->() };
 		if ( $@ ) {
 			# Handle filter errors
 			$self->{errstr} = "Filter error: $@";
@@ -340,7 +358,7 @@ sub _getline {
 
 =head2 row
 
-The C<row> method returns the current row of the CSV file.
+The C<row> method returns the line number of the most-recently-read row of the CSV file.
 
 This is a one-based count, so when you first create the parser,
 the value of C<row> will be zero (unless you are using
@@ -376,11 +394,19 @@ sub combine {
 The C<string> method is provided as a convenience, and is passed through
 to the underlying L<Text::CSV_XS> object.
 
+NOTE: relying on L<string> to return the current data after C<fetch()>
+is deprecated and will (probably) be removed in a future release.
+Only rely on its value after C<combine()>.  See similar warnings in
+L<fetch()> and L<fields()>.
+
 =cut
 
 sub string {
 	my $self = shift;
-	$self->{csv_xs}->combine(@{$self->{savedrow}});
+	if ($self->{savedrow}) {
+	  $self->combine(@{$self->{savedrow}});
+	  delete $self->{savedrow};
+	}
 	$self->{csv_xs}->string;
 }
 
@@ -408,10 +434,20 @@ sub print {
 The C<fields> method is provided as a convenience, and is passed through
 to the underlying L<Text::CSV_XS> object. It shows the actual row as an array.
 
+NOTE: relying on L<fields> to return the current data after C<fetch()>
+is deprecated and will (probably) be removed in a future release.
+Only rely on its value after C<combine()>.  See similar warnings in
+L<fetch()> and L<string()>.
+
 =cut
 
 sub fields {
-	@{shift()->{savedrow} || []};
+	my $self = shift;
+	if ($self->{savedrow}) {
+	  $self->combine(@{$self->{savedrow}});
+	  delete $self->{savedrow};
+	}
+	$self->{csv_xs}->fields;
 }
 
 =pod
